"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Analytics = void 0;
const util_1 = require("@marsupialtail/util");
const mixpanel_1 = require("mixpanel");
class Analytics {
    /**
     * Constructs an analytics object.
     * @param filename The current file taken via `new Analytics(__filename)`.
     */
    constructor(mixpanelToken, filename) {
        /**
         * Tracks an event.
         * @param name The name of the event.
         * @param user The user the event is attached to. Use null if anonymous.
         * @param event The event object.
         */
        this.event = (name, user, event) => {
            this.tryRegister(user);
            if (event) {
                const eventCopy = (0, util_1.deepCopy)(event);
                eventCopy.eventName = name;
                eventCopy.firedAt = Date.now();
                this.analytics.track(this.getEventName(name), Analytics.parseEvent(eventCopy, user));
            }
            else {
                this.analytics.track(this.getEventName(name));
            }
        };
        /**
         * Marks experiment exposure, allowing analytics to be segmented by variant.
         * @param experimentName The name of the experiment.
         * @param experimentVariant The name of the experiment variant the user is a part of.
         * @param user The user to mark exposure for.
         */
        this.markExposure = (experimentName, experimentVariant, user) => {
            this.tryRegister(user);
            this.analytics.track('$experiment_started', {
                'Experiment name': experimentName,
                'Variant name': experimentVariant,
            });
        };
        /**
         * Gets the namespaced name of the event.
         * @param name The base event name.
         * @returns A namespaced event name (ex. UserCreatedEvent -> endpoints.user.UserCreatedEvent).
         */
        this.getEventName = (name) => (this.namespace ? `${this.namespace}.${name}` : name);
        /**
         * Registers a user for in Mixpanel with a distinct_id.
         * @param user The user to register.
         */
        this.tryRegister = (user) => {
            if (user) {
                this.analytics.people.set(user.id, {
                    id: user.id,
                    username: user.username,
                    firstName: user.firstName,
                    lastName: user.lastName,
                    email: user.email,
                });
            }
        };
        this.namespace = Analytics.parseNamespace(filename);
        if (!mixpanelToken) {
            console.error('Missing analytics token');
        }
        this.analytics = (0, mixpanel_1.init)(mixpanelToken);
    }
}
exports.Analytics = Analytics;
/**
 * Parses an optional filename into a namespace. Namespaces are used
 * to tell from where an event was fired.
 * @param filename The __filename of the analytics sender.
 * @returns A parsed namespace in.this.format or null.
 */
Analytics.parseNamespace = (filename) => {
    if (!filename) {
        return null;
    }
    if (filename.includes('src')) {
        filename = filename.substring(filename.indexOf('src'));
    }
    else if (filename.includes('build')) {
        filename = filename.substring(filename.indexOf('build'));
    }
    else {
        return null;
    }
    try {
        const filenamePieces = util_1.Strings.replaceAll(filename.trim(), '.', '/')
            .split('/')
            .filter(Boolean);
        filenamePieces.shift();
        filenamePieces.pop();
        return filenamePieces.join('.');
    }
    catch (_a) {
        return null;
    }
};
/**
 * Parses a base event into an object literal with string keys.
 * @param baseEvent The event to parse.
 * @param user The user associated with this event.
 * @returns A parsed event or undefined if the event was not present.
 */
Analytics.parseEvent = (baseEvent, user) => {
    if ((0, util_1.isNullish)(baseEvent) || !(0, util_1.isObject)(baseEvent)) {
        return undefined;
    }
    const event = {};
    for (const key in baseEvent) {
        if ((0, util_1.isString)(key)) {
            event[key] = baseEvent[key];
        }
        else {
            event[String(key)] = baseEvent[key];
        }
    }
    if (user) {
        // Mixpanel links events with a 'distinct_id' to users with that user ID
        event.distinct_id = user.id;
    }
    return event;
};
//# sourceMappingURL=analytics.js.map