"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Response = void 0;
const util_1 = require("@marsupialtail/util");
class Response {
    constructor(req, res, callback) {
        this.sendResponse = (code, body, doRunCallback = true) => {
            if (!this.lock) {
                if ((0, util_1.isObject)(body) || (0, util_1.isArray)(body)) {
                    this.res.status(code).json(body);
                    this.message = JSON.stringify(body);
                }
                else if ((0, util_1.isString)(body)) {
                    this.res.status(code).send(body);
                    this.message = body;
                }
                else {
                    this.res.sendStatus(code);
                }
                this.lock = true;
                this.code = code;
                doRunCallback && this.callback && this.callback(code, body);
            }
        };
        /**
         * Assert the presence of a value for parameter testing.
         * Sends a "400 Bad Request" if the check fails.
         * @param key The key in the request object that is being tested. Split with dots to access
         * child properties of an object, such as `event.eventName`.
         * @param type The type of the value to assert.
         * @param isRequired Is the parameter required?
         * @returns True if the assertion passes, false otherwise.
         */
        this.assert = (key, type, isRequired = false) => {
            let value = null;
            if ((0, util_1.isString)(key)) {
                const splitKey = key.split('.');
                value = this.req[splitKey[0]];
                for (let i = 1; i < splitKey.length; i++) {
                    if ((0, util_1.hasKey)(value, splitKey[i])) {
                        value = value[splitKey[i]];
                    }
                    else {
                        return !isRequired;
                    }
                }
            }
            else {
                value = this.req[key];
            }
            if (isRequired || (!isRequired && !(0, util_1.isNullish)(value))) {
                if ((type === 'array' && !(0, util_1.isArray)(value)) ||
                    (type === 'boolean' && !(0, util_1.isBoolean)(value)) ||
                    (type === 'function' && !(0, util_1.isFunction)(value)) ||
                    (type === 'number' && !(0, util_1.isNumber)(value)) ||
                    (type === 'object' && !(0, util_1.isObject)(value)) ||
                    (type === 'string' && !(0, util_1.isString)(value))) {
                    this.badRequest(`${key} is not a valid ${type}`);
                    return false;
                }
            }
            return true;
        };
        // Bare responses:
        this.continue = (body) => {
            this.sendResponse(util_1.STATUS.CODE_100_CONTINUE, body);
        };
        this.ok = (body) => {
            this.sendResponse(util_1.STATUS.CODE_200_OK, body);
        };
        this.noContent = (body) => {
            this.sendResponse(util_1.STATUS.CODE_204_NO_CONTENT, body);
        };
        this.movedPermanently = (body) => {
            this.sendResponse(util_1.STATUS.CODE_301_MOVED_PERMANENTLY, body);
        };
        this.badRequest = (body) => {
            this.sendResponse(util_1.STATUS.CODE_400_BAD_REQUEST, body);
        };
        this.unauthorized = (body) => {
            this.sendResponse(util_1.STATUS.CODE_401_UNAUTHORIZED, body);
        };
        this.paymentRequired = (body) => {
            this.sendResponse(util_1.STATUS.CODE_402_PAYMENT_REQUIRED, body);
        };
        this.forbidden = (body) => {
            this.sendResponse(util_1.STATUS.CODE_403_FORBIDDEN, body);
        };
        this.notFound = (body) => {
            this.sendResponse(util_1.STATUS.CODE_404_NOT_FOUND, body);
        };
        this.methodNotAllowed = (body) => {
            this.sendResponse(util_1.STATUS.CODE_405_METHOD_NOT_ALLOWED, body);
        };
        this.notAcceptable = (body) => {
            this.sendResponse(util_1.STATUS.CODE_406_NOT_ACCEPTABLE, body);
        };
        this.requestTimeout = (body) => {
            this.sendResponse(util_1.STATUS.CODE_408_REQUEST_TIMEOUT, body);
        };
        this.conflict = (body) => {
            this.sendResponse(util_1.STATUS.CODE_409_CONFLICT, body);
        };
        this.gone = (body) => {
            this.sendResponse(util_1.STATUS.CODE_410_GONE, body);
        };
        this.tooManyRequests = (body) => {
            this.sendResponse(util_1.STATUS.CODE_429_TOO_MANY_REQUESTS, body);
        };
        this.internalServerError = (body) => {
            this.sendResponse(util_1.STATUS.CODE_500_INTERNAL_SERVER_ERROR, body);
        };
        this.notImplemented = (body) => {
            this.sendResponse(util_1.STATUS.CODE_501_NOT_IMPLEMENTED, body);
        };
        this.serviceUnavailable = (body) => {
            this.sendResponse(util_1.STATUS.CODE_503_SERVICE_UNAVAILABLE, body);
        };
        this.req = req;
        this.res = res;
        this.lock = false;
        this.callback = callback;
    }
    get isLocked() {
        return this.lock;
    }
    get statusCode() {
        return this.code;
    }
    get statusMessage() {
        return this.message;
    }
}
exports.Response = Response;
//# sourceMappingURL=response.js.map